@@ -1,7 +1,7 @@
 require 'spec_helper'
 
 describe Gitlab::Workhorse, lib: true do
-  let(:project)    { create(:project) }
+  let(:project)    { create(:project, :repository) }
   let(:repository) { project.repository }
 
   def decode_workhorse_header(array)
@@ -42,7 +42,8 @@ describe Gitlab::Workhorse, lib: true do
       out = {
         subprotocols: ['foo'],
         url: 'wss://example.com/terminal.ws',
-        headers: { 'Authorization' => ['Token x'] }
+        headers: { 'Authorization' => ['Token x'] },
+        max_session_time: 600
       }
       out[:ca_pem] = ca_pem if ca_pem
       out
@@ -53,7 +54,8 @@ describe Gitlab::Workhorse, lib: true do
         'Terminal' => {
           'Subprotocols' => ['foo'],
           'Url' => 'wss://example.com/terminal.ws',
-          'Header' => { 'Authorization' => ['Token x'] }
+          'Header' => { 'Authorization' => ['Token x'] },
+          'MaxSessionTime' => 600
         }
       }
       out['Terminal']['CAPem'] = ca_pem if ca_pem
@@ -174,4 +176,27 @@ describe Gitlab::Workhorse, lib: true do
       described_class.verify_api_request!(headers)
     end
   end
+
+  describe '.git_http_ok' do
+    let(:user) { create(:user) }
+
+    subject { described_class.git_http_ok(repository, user) }
+
+    it { expect(subject).to eq({ GL_ID: "user-#{user.id}", RepoPath: repository.path_to_repo }) }
+
+    context 'when Gitaly socket path is present' do
+      let(:gitaly_socket_path) { '/tmp/gitaly.sock' }
+
+      before do
+        allow(Gitlab.config.gitaly).to receive(:socket_path).and_return(gitaly_socket_path)
+      end
+
+      it 'includes Gitaly params in the returned value' do
+        expect(subject).to include({
+          GitalyResourcePath: "/projects/#{repository.project.id}/git-http/info-refs",
+          GitalySocketPath: gitaly_socket_path,
+        })
+      end
+    end
+  end
 end
