@@ -5,14 +5,14 @@ describe AvatarUploader do
   subject { described_class.new(user) }
 
   describe '#move_to_cache' do
-    it 'is true' do
-      expect(subject.move_to_cache).to eq(true)
+    it 'is false' do
+      expect(subject.move_to_cache).to eq(false)
     end
   end
 
   describe '#move_to_store' do
-    it 'is true' do
-      expect(subject.move_to_store).to eq(true)
+    it 'is false' do
+      expect(subject.move_to_store).to eq(false)
     end
   end
 end
